#extension GL_OES_EGL_image_external : require
#ifdef EXTERNAL_MASK
uniform samplerExternalOES mask_texture;
#else
uniform sampler2D mask_texture; // the mask texture object
#endif
uniform sampler2D shape_texture; // the shape texture object

varying highp vec2 v_brushTextureCoordinate; // texture coordinates
varying highp vec2 v_canvasTextureCoordinate; // texture coordinates

uniform highp vec4 brush_color;

uniform highp float pressure;

#ifdef USES_TEXTURE
uniform sampler2D texture_texture; // the texture texture object
varying highp vec2 v_textureTextureCoordinate; // texture coordinates
#endif

// for all settings 0 is no adjustment
lowp vec3 BrighnessContrastSaturation(lowp vec3 color, lowp float brt, lowp float con, lowp float sat)
{
    // Increase or decrease theese values to adjust r, g and b color channels seperately
    const lowp float AvgLumR = 0.5;
    const lowp float AvgLumG = 0.5;
    const lowp float AvgLumB = 0.5;

    const lowp vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);

    lowp vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
    lowp vec3 brtColor = color * (brt + 1.0); // multiply the brightness through

    // caldulate the saturation using the intensity
    lowp vec3 intensity = vec3(dot(brtColor, LumCoeff));
    lowp vec3 satColor = mix(intensity, brtColor, sat + 1.0);

    lowp vec3 conColor = mix(AvgLumin, satColor, con + 1.0);
    return clamp(conColor, 0.0, 1.0);
}

void main() {

    highp vec4 maskTexelColor = texture2D(mask_texture, v_canvasTextureCoordinate);
    // brush shape texture
    highp vec4 shapeTexelColor = texture2D(shape_texture, v_brushTextureCoordinate);
    // mask = 1 -> selected shape. keep visible
    highp float alpha =  maskTexelColor.a * pressure;


#ifdef USES_TEXTURE
    // e.g paper texture
    highp vec4 textureTexelColor = texture2D(texture_texture, v_textureTextureCoordinate);


#endif

    // imimates a light source coming from top.
    // brushes at top of canvas are brighter
    //highp float ygradient = ((1.0-v_canvasTextureCoordinate.y) * .6) + .7;

    // multiply brush color, brush shape texture, and alpha.
    // -> if brush shape has no alpha, pixels get hidden
    gl_FragColor = brush_color * shapeTexelColor.a * alpha;
    //gl_FragColor.rgb = gl_FragColor.rgb * ygradient;


#ifdef USES_TEXTURE

    // r channel of shape for brightness
    gl_FragColor = vec4(BrighnessContrastSaturation(gl_FragColor.rgb, textureTexelColor.r * 2.0, 0.0, 0.0), gl_FragColor.a);

#endif

}